//------------------------------------------------------------
//  Project: Open Tools for Xbase++                          -
//  Repository : http://www.xbwin.com                        -
//  Author: Pablo Botella Navarro ( http://www.xbwin.com )   -
//------------------------------------------------------------
#include "ot4xb.ch"
#include "sqlite3_xb.ch"
//----------------------------------------------------------------------------------------------------------------------
CLASS TSqLiteDb
EXPORTED:
       // ---------------------------------------------------------------------------------
        VAR hdb
        VAR nLastError
       // ---------------------------------------------------------------------------------
INLINE CLASS METHOD initClass()
       nLoadLibrary("sqlite3")
       return Self
       // ---------------------------------------------------------------------------------
INLINE CLASS METHOD Open( cDb )
       local hdb      := 0
       local peCode   := 0
       local result,s
       DEFAULT cDb := ""
       result := @sqlite3:sqlite3_open(cAnsiToUtf8(cDb) , @hdb)
       if result == 0
          s := Self
          return s:New(hdb)
       end
       return NIL
       // ---------------------------------------------------------------------------------
INLINE METHOD init(hdb)
       BYNAME hdb
       Return Self
       // ---------------------------------------------------------------------------------
INLINE METHOD Close()
       local result := NIL
       if !Empty(::hdb)
          result := @sqlite3:sqlite3_close(::hdb)
          ::hdb := 0
       end
       return result == 0
       // ---------------------------------------------------------------------------------
INLINE ACCESS METHOD aTableNames
       local aa := ::exec("SELECT name FROM sqlite_master WHERE type='table' ORDER BY name;")
       local result := Array(Len(aa))
       aEval( aa , {|e,n| result[n] := e[1] } )
       return result
       // ---------------------------------------------------------------------------------
INLINE ACCESS METHOD lIsTable( cTable )
       local aa := ::exec(cPrintf("SELECT name FROM sqlite_master WHERE type='table' AND name='%s' COLLATE NOCASE",cTable))
       return Len(aa) > 0
       // ---------------------------------------------------------------------------------
INLINE ACCESS METHOD lIsTrigger( cTrigger )
       local aa := ::exec(cPrintf("SELECT name FROM sqlite_master WHERE type='trigger' AND name='%s' COLLATE NOCASE",cTrigger))
       return Len(aa) > 0
       // ---------------------------------------------------------------------------------
INLINE METHOD exec(cSql,aCol,nLastError) // -> aResult | NIL
       local result,nColCount,pStm
       local nRetry  := 0
       local aRows := NIL
       local nRow,nCol
       ::nLastError := nLastError := 0 
       if Empty(cSql) ; return NIL ; end
       pStm := 0
       result := @sqlite3:sqlite3_prepare_v2(::hdb,cSql,-1,@pStm,0)
       if( result != SQLITE_OK ) ; ::nLastError := result ; return NIL ; end
       nColCount := @sqlite3:sqlite3_column_count(pStm)
       aCol := Array(nColCount)
       aEval(aCol , {|e,n| aCol[n] := PeekStr(@sqlite3:sqlite3_column_name(pStm,n-1),,-1) })
       aRows := {} ; result := 0 ; nRow := 0
       while( (result := @sqlite3:sqlite3_step(pStm)) == SQLITE_ROW )
          aadd( aRows , Array(nColCount) ) ; nRow++
          for nCol := 1 to nColCount
            aRows[nRow][nCol] := PeekStr(@sqlite3:sqlite3_column_text(pStm,nCol-1),,-1)
          next
       end
       ::nLastError := nLastError := result
       @sqlite3:sqlite3_finalize(pStm)
       return aRows
       // ---------------------------------------------------------------------------------
INLINE METHOD sql() // -> aResult | NIL
       local n
       local nn   := PCount()
       local ap   := Array(nn)
       local cSql := ""
       for n := 1 to nn ; ap[n] := PValue(n) ; next
       lCallFuncPA( "cPrintf",ap,@cSql)
       return ::exec(cAnsiToUtf8(cSql))
       // ---------------------------------------------------------------------------------
INLINE METHOD ExecA(cSql,aCol,nLastError) // -> aResult | NIL
       if Empty(cSql) ; return NIL ; end
       return ::exec(cAnsiToUtf8(cSql),@aCol,@nLastError)
       // ---------------------------------------------------------------------------------
INLINE METHOD ExecO(cSql,aCol,nLastError) // -> aResult | NIL
       if Empty(cSql) ; return NIL ; end
       return ::exec(cOemToUtf8(cSql),@aCol,@nLastError)
       // ---------------------------------------------------------------------------------
INLINE METHOD exec_begin(cSql,nLastError) // -> pStm | 0
       local result,pStm
       ::nLastError := nLastError := 0
       if Empty(cSql) ; return NIL ; end
       pStm := 0
       ::nLastError := nLastError := @sqlite3:sqlite3_prepare_v2(::hdb,cSql,-1,@pStm,0)
       if( nLastError != SQLITE_OK ) ; return NIL ; end
       return pStm
       // ---------------------------------------------------------------------------------
INLINE METHOD exec_beginA(cSql,nLastError) // -> pStm | 0
       local result,pStm
       ::nLastError := nLastError := 0
       if Empty(cSql) ; return NIL ; end
       pStm := 0
       ::nLastError := nLastError := @sqlite3:sqlite3_prepare_v2(::hdb,cAnsiToUtf8(cSql),-1,@pStm,0)
       if( nLastError != SQLITE_OK ) ; return NIL ; end
       return pStm
       // ---------------------------------------------------------------------------------
       // ---------------------------------------------------------------------------------
INLINE METHOD exec_end(pStm,aCol,nLastError) // -> aResult | NIL
       local result,nColCount
       local nRetry  := 0
       local aRows := NIL
       local nRow,nCol
       ::nLastError := nLastError := 0
       if Empty(pStm) ; return NIL ; end
       nColCount := @sqlite3:sqlite3_column_count(pStm)
       aCol := Array(nColCount)
       aEval(aCol , {|e,n| aCol[n] := PeekStr(@sqlite3:sqlite3_column_name(pStm,n-1),,-1) })
       aRows := {} ; result := 0 ; nRow := 0
       while .T.
          result := @sqlite3:sqlite3_step(pStm)
          if( result == SQLITE_ROW )
             aadd( aRows , Array(nColCount) ) ; nRow++
             for nCol := 1 to nColCount
               aRows[nRow][nCol] := PeekStr(@sqlite3:sqlite3_column_text(pStm,nCol-1),,-1)
             next
          elseif( result == SQLITE_DONE)
             exit
          else
             ? "exec_end->" , result
             EXIT
          end
       end
       ::nLastError := nLastError := result
       @sqlite3:sqlite3_finalize(pStm)
       return aRows
       // ---------------------------------------------------------------------------------
INLINE METHOD bind_blob(pStm,k,v)
       local n
       if Empty(pStm) ; return NIL ; end
       n := @sqlite3:sqlite3_bind_parameter_index(pStm,k)
       DEFAULT v := ""
       return @sqlite3:sqlite3_bind_blob(pStm,n,v,Len(v),-1)
       // ---------------------------------------------------------------------------------
INLINE METHOD bind_int(pStm,k,v)
       local n
       if Empty(pStm) ; return NIL ; end
       n := @sqlite3:sqlite3_bind_parameter_index(pStm,k)
       return @sqlite3:sqlite3_bind_int(pStm,n,v)
       // ---------------------------------------------------------------------------------
INLINE METHOD bind_double(pStm,k,v)
       local n
       if Empty(pStm) ; return NIL ; end
       n := @sqlite3:sqlite3_bind_parameter_index(pStm,k)
       return @sqlite3:[__sl__sl__sl__f8]:sqlite3_bind_double(pStm,n,v)
       // ---------------------------------------------------------------------------------
INLINE METHOD bind_int64(pStm,k,v)
       local n
       if Empty(pStm) ; return NIL ; end
       n := @sqlite3:sqlite3_bind_parameter_index(pStm,k)
       return @sqlite3:[__sl__sl__sl__sq]:sqlite3_bind_int64(pStm,n,v)
       // ---------------------------------------------------------------------------------
INLINE METHOD bind_null(pStm,k)
       local n
       if Empty(pStm) ; return NIL ; end
       n := @sqlite3:sqlite3_bind_parameter_index(pStm,k)
       return @sqlite3:sqlite3_bind_null(pStm,n)
       // ---------------------------------------------------------------------------------
INLINE METHOD bind_textO(pStm,k,v)
       local n
       if Empty(pStm) ; return NIL ; end
       n := @sqlite3:sqlite3_bind_parameter_index(pStm,k)
       DEFAULT v := ""
       return @sqlite3:sqlite3_bind_text(pStm,n,v,Len(v),-1)
       // ---------------------------------------------------------------------------------
INLINE METHOD bind_textA(pStm,k,v)
       local va
       local n
       if Empty(pStm) ; return NIL ; end
       n := @sqlite3:sqlite3_bind_parameter_index(pStm,k)
       DEFAULT v := ""
       va := cAnsiToUtf8(v)
       return @sqlite3:sqlite3_bind_text(pStm,n,va,Len(va),-1)
       // ---------------------------------------------------------------------------------
INLINE METHOD bind_zeroblob(pStm,k,v)
       local n
       if Empty(pStm) ; return NIL ; end
       n := @sqlite3:sqlite3_bind_parameter_index(pStm,k)
       return @sqlite3:sqlite3_bind_zeroblob(pStm,n,v)
       // ---------------------------------------------------------------------------------

ENDCLASS
//----------------------------------------------------------------------------------------------------------------------
